@@ -14,9 +14,11 @@
 #include "absl/log/absl_log.h"
 #include "absl/log/die_if_null.h"
 #include "absl/strings/str_cat.h"
+#include "absl/strings/str_format.h"
 #include "conformance_test.h"
-#include "google/protobuf/editions/golden/test_messages_proto2_editions.pb.h"
-#include "google/protobuf/editions/golden/test_messages_proto3_editions.pb.h"
+#include "conformance/test_protos/test_messages_edition2023.pb.h"
+#include "editions/golden/test_messages_proto2_editions.pb.h"
+#include "editions/golden/test_messages_proto3_editions.pb.h"
 #include "google/protobuf/test_messages_proto2.pb.h"
 #include "google/protobuf/test_messages_proto3.pb.h"
 #include "google/protobuf/text_format.h"
@@ -24,6 +26,7 @@
 using conformance::ConformanceRequest;
 using conformance::ConformanceResponse;
 using conformance::WireFormat;
+using protobuf_test_messages::editions::TestAllTypesEdition2023;
 using protobuf_test_messages::proto2::TestAllTypesProto2;
 using protobuf_test_messages::proto2::UnknownToTestAllTypes;
 using protobuf_test_messages::proto3::TestAllTypesProto3;
@@ -122,6 +125,7 @@ void TextFormatConformanceTestSuite::RunSuiteImpl() {
   if (maximum_edition_ >= Edition::EDITION_2023) {
     TextFormatConformanceTestSuiteImpl<TestAllTypesProto2Editions>(this);
     TextFormatConformanceTestSuiteImpl<TestAllTypesProto3Editions>(this);
+    TextFormatConformanceTestSuiteImpl<TestAllTypesEdition2023>(this);
   }
 }
 
@@ -131,11 +135,18 @@ TextFormatConformanceTestSuiteImpl<MessageType>::
     : suite_(*ABSL_DIE_IF_NULL(suite)) {
   // Flag control performance tests to keep them internal and opt-in only
   if (suite_.performance_) {
+    if (MessageType::GetDescriptor()->name() == "TestAllTypesEdition2023") {
+      // There are no editions-sensitive performance tests.
+      return;
+    }
     RunTextFormatPerformanceTests();
   } else {
     if (MessageType::GetDescriptor()->name() == "TestAllTypesProto2") {
       RunGroupTests();
     }
+    if (MessageType::GetDescriptor()->name() == "TestAllTypesEdition2023") {
+      RunDelimitedTests();
+    }
     if (MessageType::GetDescriptor()->name() == "TestAllTypesProto3") {
       RunAnyTests();
       // TODO Run these over proto2 also.
@@ -235,6 +246,33 @@ void TextFormatConformanceTestSuiteImpl<
   suite_.RunValidBinaryInputTest(setting2, serialized_input);
 }
 
+template <typename MessageType>
+void TextFormatConformanceTestSuiteImpl<MessageType>::RunDelimitedTests() {
+  RunValidTextFormatTest("GroupFieldNoColon", REQUIRED,
+                         "GroupLikeType { group_int32: 1 }");
+  RunValidTextFormatTest("GroupFieldWithColon", REQUIRED,
+                         "GroupLikeType: { group_int32: 1 }");
+  RunValidTextFormatTest("GroupFieldEmpty", REQUIRED, "GroupLikeType {}");
+  RunValidTextFormatTest(
+      "GroupFieldExtension", REQUIRED,
+      "[protobuf_test_messages.editions.groupliketype] { c: 1 }");
+  RunValidTextFormatTest(
+      "DelimitedFieldExtension", REQUIRED,
+      "[protobuf_test_messages.editions.delimited_ext] { c: 1 }");
+
+  // Test that lower-cased group name (i.e. implicit field name) are accepted.
+  RunValidTextFormatTest("DelimitedFieldLowercased", REQUIRED,
+                         "groupliketype { group_int32: 1 }");
+  RunValidTextFormatTest("DelimitedFieldLowercasedDifferent", REQUIRED,
+                         "delimited_field { group_int32: 1 }");
+
+  // Extensions always used the field name, and should never accept the message
+  // name.
+  ExpectParseFailure(
+      "DelimitedFieldExtensionMessageName", REQUIRED,
+      "[protobuf_test_messages.editions.GroupLikeType] { group_int32: 1 }");
+}
+
 template <typename MessageType>
 void TextFormatConformanceTestSuiteImpl<MessageType>::RunGroupTests() {
   RunValidTextFormatTest("GroupFieldNoColon", REQUIRED,
@@ -242,6 +280,28 @@ void TextFormatConformanceTestSuiteImpl<MessageType>::RunGroupTests() {
   RunValidTextFormatTest("GroupFieldWithColon", REQUIRED,
                          "Data: { group_int32: 1 }");
   RunValidTextFormatTest("GroupFieldEmpty", REQUIRED, "Data {}");
+  RunValidTextFormatTest("GroupFieldMultiWord", REQUIRED,
+                         "MultiWordGroupField { group_int32: 1 }");
+
+  // Test that lower-cased group name (i.e. implicit field name) is accepted
+  RunValidTextFormatTest("GroupFieldLowercased", REQUIRED,
+                         "data { group_int32: 1 }");
+  RunValidTextFormatTest("GroupFieldLowercasedMultiWord", REQUIRED,
+                         "multiwordgroupfield { group_int32: 1 }");
+
+  // Test extensions of group type
+  RunValidTextFormatTest("GroupFieldExtension", REQUIRED,
+                         absl::StrFormat("[%s] { group_int32: 1 }",
+                                         MessageType::GetDescriptor()
+                                             ->file()
+                                             ->FindExtensionByName("groupfield")
+                                             ->PrintableNameForExtension()));
+  ExpectParseFailure("GroupFieldExtensionGroupName", REQUIRED,
+                     absl::StrFormat("[%s] { group_int32: 1 }",
+                                     MessageType::GetDescriptor()
+                                         ->file()
+                                         ->FindMessageTypeByName("GroupField")
+                                         ->full_name()));
 }
 
 template <typename MessageType>
