@@ -155,7 +155,7 @@ void EmitLinkWKTs(absl::string_view name, io::Printer* p) {
             // This is to help make sure that the GPBWellKnownTypes.* categories get linked and
             // developers do not have to use the `-ObjC` linker flag. More information
             // here: https://medium.com/ios-os-x-development/categories-in-static-libraries-78e41f8ddb96
-            __attribute__((used)) static NSString* $basename$_importCategories () {
+            __attribute__((used)) static NSString* $basename$_importCategories(void) {
               return GPBWellKnownTypesErrorDomain;
             }
           )objc");
@@ -598,6 +598,10 @@ void FileGenerator::GenerateFile(io::Printer* p, GeneratedFileType file_type,
 
   p->Emit(
       {
+          {"no_checked_in",
+           "NO CHECKED-IN"
+           // Intentional line breaker
+           " PROTOBUF GENCODE"},
           {"filename", file_->name()},
           {"google_protobuf_objc_version", GOOGLE_PROTOBUF_OBJC_VERSION},
           {"runtime_imports",
@@ -632,6 +636,7 @@ void FileGenerator::GenerateFile(io::Printer* p, GeneratedFileType file_type,
       },
       R"objc(
         // Generated by the protocol buffer compiler.  DO NOT EDIT!
+        // $no_checked_in$
         // $clangfmt$ off
         // source: $filename$
 
